#pragma once
#include <rapidjson/document.h>

#include "utils.h"

inline struct session_data {
    std::string session_cookie;
    std::string bmc_ip;
    std::string csrf_token;
    static session_data from_json(const rapidjson::Value& obj) {
        return {
            .session_cookie = obj["SESSION_COOKIE"].GetString(),
            .bmc_ip = obj["BMC_IP_ADDR"].GetString(),
            .csrf_token = obj["CSRFTOKEN"].GetString()
        };
    }
} session;

enum class fan_status {
    critical     = 0, // 严重
    non_critical = 1, // 正常
    absent       = 2, // 不可用
};

struct fan_info {
    int id = -1;
    int present = -1;
    fan_status status = fan_status::absent;
    int speed_percent = -1;
    int speed_rpm = -1;
    static fan_info from_json(const rapidjson::Value& obj) {
        fan_info out{
            .id = obj["Id"].GetInt(),
            .present = obj["Present"].GetInt(),
            .speed_percent = obj["SpeedPercent"].GetInt(),
            .speed_rpm = obj["SpeedRPM"].GetInt()
        };
        const int s = obj["Status"].GetInt();
        if (s == 2) {
            out.status = fan_status::absent;
        } else if (out.speed_rpm == 0) {
            out.status = fan_status::critical;
        } else {
            out.status = fan_status::non_critical;
        }
        return out;
    }
};

struct fan_mode_info {
    bool manual_mode= false;
    static fan_mode_info from_json(const rapidjson::Value& obj) {
        return {
            .manual_mode = obj["fanMode"].GetInt() == 1
        };
    }
};

struct running_time {
    int days = -1;
    int hours = -1;
    static running_time from_json(const rapidjson::Value& obj) {
        return {
            .days = obj["Days"].GetInt(),
            .hours = obj["Hours"].GetInt(),
        };
    }
};

struct psu_status {
    int id = -1;
    int present = -1;
    int power_status = -1;
    int temperature = -1;
    int pwr_in_watts = -1;
    int input_power = -1;
    int input_volt = -1;
    int output_volt = -1;
    int input_current = -1;
    int output_current = -1;
    int err_status = -1;
    std::string fw_version = "unknown";
    int output_power_max = -1;
    std::string mfr_model = "unknown";
    std::string mfr_id = "unknown";
    std::string sn = "unknown";

    static psu_status from_json(const rapidjson::Value& obj) {
        return {
            .id = obj["Id"].GetInt(),
            .present = obj["Present"].GetInt(),
            .power_status = obj["PowerStatus"].GetInt(),
            .temperature = obj["Temperature"].GetInt(),
            .pwr_in_watts = obj["PwrInWatts"].GetInt(),
            .input_power = obj["InputPower"].GetInt(),
            .input_volt = obj["InputVolt"].GetInt(),
            .output_volt = obj["OutputVolt"].GetInt(),
            .input_current = obj["InputCurrent"].GetInt(),
            .output_current = obj["OutputCurrent"].GetInt(),
            .err_status = obj["ErrStatus"].GetInt(),
            .fw_version = obj["FWVersion"].GetString(),
            .output_power_max = obj["OutputPowerMax"].GetInt(),
            .mfr_model = obj["MFRModel"].GetString(),
            .mfr_id = obj["MFRID"].GetString(),
            .sn = obj["SN"].GetString()
        };
    }
};
