#pragma once
#include <iomanip>
#include <regex>
#include <set>
#include <string>
#include <bits/ranges_algo.h>

// 定义包含需要编码的字符的集合
const std::set<char> CUSTOM_ENCODE_SET = {
    ' ', '!', '"', '#', '$', '%', '&', '\'', '(', ')', '*', '+',
    ',', '/', ':', ';', '<', '=', '>', '?', '@', '[', '\\', ']',
    '^', '`', '{', '|', '}', '~'
};

// URL编码函数
inline std::string encode_content(const std::string &value) {
    std::ostringstream escaped;
    escaped.fill('0');
    escaped << std::hex;

    for (char c : value) {
        // 如果字符在自定义的编码集合中，进行百分号编码
        if (CUSTOM_ENCODE_SET.contains(c) || !isprint(c)) {
            escaped << '%' << std::setw(2) << int((unsigned char)c);
        } else {
            escaped << c;
        }
    }

    return escaped.str();
}

inline std::string parser_js(const std::string& in_js) {
    // 第一个正则表达式
    // std::regex re(R"(\[\s*\{[^{}]*\}\s*,\s*\{[^{}]*\}\s*\])");
    // std::regex re(R"(\[.*?\])", std::regex::dotall);
    std::regex bracket_regex(R"(\[([\s\S]*?)\])");
    std::smatch match;

    if (std::regex_search(in_js, match, bracket_regex)) {
        std::string matched_str = match.str();
        // remove ",  {}"
        if (auto f = matched_str.find(",  {}"); f != std::string::npos)
            matched_str = matched_str.erase(f, 5);
        // replace ' with "
        std::ranges::replace(matched_str, '\'', '\"');
        return matched_str;
    }
    return "";
}